import pytest
from projects_organizer import app
from typer.testing import CliRunner


def test_show(projects_dir):
    pfilename = "project1"
    runner = CliRunner()
    test_args = ["-d", projects_dir, "show", pytest.projects[pfilename]["title"]]
    result = runner.invoke(app, test_args)
    assert result.exit_code == 0
    assert result.stdout == str(pytest.projects[pfilename]) + "\n"


def test_show_not_found(projects_dir):
    runner = CliRunner(mix_stderr=False)
    test_args = ["-d", projects_dir, "show", "not_found"]
    result = runner.invoke(app, test_args)
    assert result.exit_code == 1
    assert result.stdout == ""


def test_show_many_found(projects_dir):
    runner = CliRunner(mix_stderr=False)
    test_args = ["-d", projects_dir, "show", "project"]
    result = runner.invoke(app, test_args)
    assert result.exit_code == 1
    assert result.stdout == "Many projects found: Project 1, Project 2, Project 3\n"


def test_show_verbose(projects_dir):
    runner = CliRunner(mix_stderr=False)
    test_args = ["-d", projects_dir, "-v", "show", "project 1"]
    result = runner.invoke(app, test_args)
    assert result.exit_code == 0
    lines = result.stdout.strip().split("\n")
    assert "'title': 'project 1'" in lines[len(lines) - 1].lower()



from projects_organizer import app
from typer.testing import CliRunner


def test_validate(projects_dir, schema_file):
    runner = CliRunner()
    test_args = ["-d", str(projects_dir), "validate", str(schema_file)]
    result = runner.invoke(app, test_args)
    assert result.exit_code == 0
    assert result.stdout == "All projects are valid.\n"


def test_validate_missing_tag(projects_dir, schema_file_missing_tag):
    runner = CliRunner()
    test_args = ["-d", str(projects_dir), "validate", str(schema_file_missing_tag)]
    result = runner.invoke(app, test_args)
    assert result.exit_code == 1
    assert result.stdout != "All projects are valid.\n"
    assert len(result.stdout.strip().split("\n")) == 2


def test_validate_missing_tag_stop(projects_dir, schema_file_missing_tag):
    runner = CliRunner()
    test_args = [
        "-d",
        str(projects_dir),
        "validate",
        str(schema_file_missing_tag),
        "-s",
    ]
    result = runner.invoke(app, test_args)
    assert result.exit_code == 1
    assert result.stdout.startswith("error for project Project 1\n")


def test_validate_invalid(projects_dir, schema_file_invalid):
    runner = CliRunner()
    test_args = ["-d", str(projects_dir), "validate", str(schema_file_invalid)]
    result = runner.invoke(app, test_args)
    assert result.exit_code == 1
    assert "invalid schema" in result.stdout.lower()



"""
This type stub file was generated by pyright.
"""

from typing import AnyStr

"""
Utilities for handling unicode and other repetitive bits
"""

def u(text: AnyStr, encoding: str = ...) -> str:
    "Return unicode text, no matter what"
    ...



import pytest
from projects_organizer import app
from typer.testing import CliRunner


def test_list_basic(projects_dir):
    runner = CliRunner()
    test_args = ["-d", projects_dir, "list"]
    result = runner.invoke(app, test_args)
    assert result.exit_code == 0
    assert result.stdout == "- Project 1\n- Project 2\n- Project 3\n"


def test_list_basic_verbose(projects_dir):
    runner = CliRunner()
    test_args = ["-d", projects_dir, "-v", "list"]
    result = runner.invoke(app, test_args)
    assert result.exit_code == 0
    assert (
        result.stdout
        == f"""Initializing projects from {str(projects_dir)}
Reading file {str(projects_dir / "project1" / "index.md")}
Reading file {str(projects_dir / "project2" / "index.md")}
Reading file {str(projects_dir / "project3" / "index.md")}
- Project 1
- Project 2
- Project 3
"""
    )


def test_list_empty_project(projects_dir_empty_project):
    runner = CliRunner()
    test_args = ["-d", projects_dir_empty_project, "list"]
    result = runner.invoke(app, test_args)
    assert result.exit_code == 1
    assert "missing index.md in" in result.stdout.lower()


def test_list_duplicate_project(projects_dir_duplicate_project):
    runner = CliRunner()
    test_args = ["-d", projects_dir_duplicate_project, "list"]
    result = runner.invoke(app, test_args)
    assert result.exit_code == 1
    assert "duplicate project title" in result.stdout.lower()


@pytest.mark.parametrize(
    "filter,expected",
    [
        ("unknown", ""),
        ("not unknown", "- Project 1\n- Project 2\n- Project 3\n"),
        ("archived", "- Project 1\n"),
        ("not archived", "- Project 2\n- Project 3\n"),
        ("'dev' in tags", "- Project 1\n- Project 2\n- Project 3\n"),
        ("'python' in tags", "- Project 1\n- Project 3\n"),
        (
            "datetime.strptime(created_at, '%Y-%m-%d') >= datetime(2024, 1, 1)",
            "- Project 2\n- Project 3\n",
        ),
    ],
)
def test_list_filter(filter, expected, projects_dir):
    runner = CliRunner()
    test_args = ["-d", projects_dir, "list", "-f", filter]
    result = runner.invoke(app, test_args)
    assert result.exit_code == 0
    assert result.stdout == expected


def test_list_filter_invalid(projects_dir):
    runner = CliRunner(mix_stderr=False)
    test_args = ["-d", projects_dir, "list", "-f", "1 + 2"]
    result = runner.invoke(app, test_args)
    assert result.exit_code != 0
    assert len(result.stderr) != 0



"""
This type stub file was generated by pyright.
"""

import io
from typing import Iterable, TYPE_CHECKING
from .default_handlers import BaseHandler

"""
Python Frontmatter: Parse and manage posts with YAML frontmatter
"""
if TYPE_CHECKING: ...
__all__ = ["parse", "load", "loads", "dump", "dumps"]
handlers = ...

def detect_format(text: str, handlers: Iterable[BaseHandler]) -> BaseHandler | None:
    """
    Figure out which handler to use, based on metadata.
    Returns a handler instance or None.

    ``text`` should be unicode text about to be parsed.

    ``handlers`` is a dictionary where keys are opening delimiters
    and values are handler instances.
    """
    ...

def parse(
    text: str,
    encoding: str = ...,
    handler: BaseHandler | None = ...,
    **defaults: object,
) -> tuple[dict[str, object], str]:
    """
    Parse text with frontmatter, return metadata and content.
    Pass in optional metadata defaults as keyword args.

    If frontmatter is not found, returns an empty metadata dictionary
    (or defaults) and original text content.

    .. testsetup:: *

        >>> import frontmatter

    .. doctest::

        >>> with open('tests/yaml/hello-world.txt') as f:
        ...     metadata, content = frontmatter.parse(f.read())
        >>> print(metadata['title'])
        Hello, world!

    """
    ...

def check(fd: str | io.IOBase, encoding: str = ...) -> bool:
    """
    Check if a file-like object or filename has a frontmatter,
    return True if exists, False otherwise.

    If it contains a frontmatter but it is empty, return True as well.

    .. doctest::

        >>> frontmatter.check('tests/yaml/hello-world.txt')
        True

    """
    ...

def checks(text: str, encoding: str = ...) -> bool:
    """
    Check if a text (binary or unicode) has a frontmatter,
    return True if exists, False otherwise.

    If it contains a frontmatter but it is empty, return True as well.

    .. doctest::

        >>> with open('tests/yaml/hello-world.txt') as f:
        ...     frontmatter.checks(f.read())
        True

    """
    ...

def load(
    fd: str | io.IOBase,
    encoding: str = ...,
    handler: BaseHandler | None = ...,
    **defaults: object,
) -> Post:
    """
    Load and parse a file-like object or filename,
    return a :py:class:`post <frontmatter.Post>`.

    .. doctest::

        >>> post = frontmatter.load('tests/yaml/hello-world.txt')
        >>> with open('tests/yaml/hello-world.txt') as f:
        ...     post = frontmatter.load(f)

    """
    ...

def loads(
    text: str,
    encoding: str = ...,
    handler: BaseHandler | None = ...,
    **defaults: object,
) -> Post:
    """
    Parse text (binary or unicode) and return a :py:class:`post <frontmatter.Post>`.

    .. doctest::

        >>> with open('tests/yaml/hello-world.txt') as f:
        ...     post = frontmatter.loads(f.read())

    """
    ...

def dump(
    post: Post,
    fd: str | io.IOBase,
    encoding: str = ...,
    handler: BaseHandler | None = ...,
    **kwargs: object,
) -> None:
    """
    Serialize :py:class:`post <frontmatter.Post>` to a string and write to a file-like object.
    Text will be encoded on the way out (utf-8 by default).

    ::

        >>> from io import BytesIO
        >>> post = frontmatter.load('tests/yaml/hello-world.txt')
        >>> f = BytesIO()
        >>> frontmatter.dump(post, f)
        >>> print(f.getvalue().decode('utf-8'))
        ---
        layout: post
        title: Hello, world!
        ---
        <BLANKLINE>
        Well, hello there, world.


    .. testcode::

        from io import BytesIO
        post = frontmatter.load('tests/yaml/hello-world.txt')
        f = BytesIO()
        frontmatter.dump(post, f)
        print(f.getvalue().decode('utf-8'))

    .. testoutput::

        ---
        layout: post
        title: Hello, world!
        ---
        <BLANKLINE>
        Well, hello there, world.

    """
    ...

def dumps(post: Post, handler: BaseHandler | None = ..., **kwargs: object) -> str:
    """
    Serialize a :py:class:`post <frontmatter.Post>` to a string and return text.
    This always returns unicode text, which can then be encoded.

    Passing ``handler`` will change how metadata is turned into text. A handler
    passed as an argument will override ``post.handler``, with
    :py:class:`YAMLHandler <frontmatter.default_handlers.YAMLHandler>` used as
    a default.

    ::

        >>> post = frontmatter.load('tests/yaml/hello-world.txt')
        >>> print(frontmatter.dumps(post)) # doctest: +NORMALIZE_WHITESPACE
        ---
        layout: post
        title: Hello, world!
        ---
        <BLANKLINE>
        Well, hello there, world.

    .. testcode::

        post = frontmatter.load('tests/yaml/hello-world.txt')
        print(frontmatter.dumps(post))

    .. testoutput::

        ---
        layout: post
        title: Hello, world!
        ---

        Well, hello there, world.

    """
    ...

class Post:
    """
    A post contains content and metadata from Front Matter. This is what gets
    returned by :py:func:`load <frontmatter.load>` and :py:func:`loads <frontmatter.loads>`.
    Passing this to :py:func:`dump <frontmatter.dump>` or :py:func:`dumps <frontmatter.dumps>`
    will turn it back into text.

    For convenience, metadata values are available as proxied item lookups.
    """
    def __init__(
        self, content: str, handler: BaseHandler | None = ..., **metadata: object
    ) -> None: ...
    def __getitem__(self, name: str) -> object:
        "Get metadata key"
        ...

    def __contains__(self, item: object) -> bool:
        "Check metadata contains key"
        ...

    def __setitem__(self, name: str, value: object) -> None:
        "Set a metadata key"
        ...

    def __delitem__(self, name: str) -> None:
        "Delete a metadata key"
        ...

    def __bytes__(self) -> bytes: ...
    def __str__(self) -> str: ...
    def get(self, key: str, default: object = ...) -> object:
        "Get a key, fallback to default"
        ...

    def keys(self) -> Iterable[str]:
        "Return metadata keys"
        ...

    def values(self) -> Iterable[object]:
        "Return metadata values"
        ...

    def to_dict(self) -> dict[str, object]:
        "Post as a dict, for serializing"
        ...
